from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import render
from django.urls import reverse
from django.utils.translation import gettext as _

from foledol.django.logger import log
from foledol.django.tools.field import TextField, IntegerField, BooleanField, DateField, FloatField
from foledol.django.tools.form import Form
from foledol.django.tools.handlers import confirm
from foledol.django.utils import pop_path, get_path, get_param, get_integer, error, new_context
from .grid import get_field
from ..models import CRITERIA_SET, TEXT_CRITERIA_SET, DATE_CRITERIA_SET, \
    NUMBER_CRITERIA_SET, BOOLEAN_CRITERIA_SET, Condition, ConditionGroup


class ConditionForm(Form):
    def __init__(self, context):
        super().__init__(context, [
            TextField('label', "Libellé", min_length=1),
            IntegerField('criteria', "Critère"),
            TextField('value', "Valeur")
        ])


def condition_form(request, condition):
    if not request.user.is_staff:
        return HttpResponseForbidden()

    context = new_context()
    context['base'] = settings.DEFAULT_SPACE + '/base.html'

    action = get_path(request, context, condition, 'condition', space='django')

    condition_group_id = get_integer(request, 'condition_group_id', 0)
    if condition_group_id > 0:
        condition_group = ConditionGroup.objects.all().get(id=condition_group_id)
    elif condition:
        condition_group = condition.condition_group
    context['condition_group_id'] = condition_group.id

    if condition:
        context['condition'] = condition

    grid = condition_group.grid() if condition_group else None
    fields = grid.fields() if condition_group else []
    context['fields'] = fields

    context['criteria_set'] = CRITERIA_SET

    form = ConditionForm(context)
    form.read(request.POST, condition)

    order = (condition_group.condition_set.count() + 1) * 10 if condition_group else 10

    criteria = get_integer(request, 'criteria', condition.criteria if condition else 0)
    context['criteria'] = criteria

    if len(action) > 0:
        form.validate()

    if context['error']:
        context['error'] = "Veuillez corriger les erreurs ci-dessous"

    label = get_param(request, context, 'label', None)
    if label and condition:
        field = get_field(condition.label, fields)
        if isinstance(field, TextField):
            if criteria not in TEXT_CRITERIA_SET:
                context['error'] = "Le critère n'est pas compatible avec un texte"
        if isinstance(field, DateField):
            if criteria not in DATE_CRITERIA_SET:
                context['error'] = "Le critère n'est pas compatible avec une date"
        if isinstance(field, FloatField) or isinstance(field, IntegerField):
            if criteria not in NUMBER_CRITERIA_SET:
                context['error'] = "Le critère n'est pas compatible avec un nombre"
        if isinstance(field, BooleanField):
            if criteria not in BOOLEAN_CRITERIA_SET:
                context['error'] = "Le critère n'est pas compatible avec un booléen"

    if not action or context['error']:
        context['action'] = 'update' if condition else 'create'
        return render(request, 'condition.html', context)

    if action == 'create':
        condition = Condition(condition_group=condition_group)
    form.save(condition)
    if not condition.order:
        condition.order = order
    condition.save()
    #if condition_group:
    #    condition_group_renumber(condition_group)
    log(condition.id, 'condition', action, request.user, form.old_values, form.new_values)

    return HttpResponseRedirect(context['back'] + '?path=' + pop_path(request))


@login_required
def condition_create(request):
    return condition_form(request, None)


@login_required
def condition_update(request, pk):
    condition = Condition.objects.filter(id=pk).first()
    return condition_form(request, condition) if condition else error(request)


@login_required
def condition_delete(request, pk):
    def prepare(context):
        condition = Condition.objects.get(id=pk)
        context['title'] = _('delete_condition_title').format(condition)
        context['cancel'] = reverse('django:condition_update', kwargs={'pk': pk})
        context['message'] = _('delete_condition_message').format(condition)
        return condition

    def execute(condition):
        condition.delete()

    return confirm(request, 'delete', prepare, execute, reverse('django:conditions'))



from io import BytesIO

from reportlab.lib.pagesizes import landscape, letter

from foledol.django.templatetags.form_extras import value
from foledol.django.tools.canvas import NumberedPageCanvas
from foledol.django.tools.report import Report, ReportColumn


class GridReport(Report):
    def __init__(self, title, grid, fields):
        self.title = title
        self.grid = grid
        self.fields = fields
        self.columns = []
        for column in self.grid.column_set.all().order_by('order'):
            field = self.get_field(column.label)
            if field:
                width = column.width if column.width and column.width > 0 else 60
                self.columns.append(
                    ReportColumn(width, field.label)
                )

    def get_field(self, key):
        for field in self.fields:
            if field.key == key:
                return field
        return None

    def header(self, page, x, y):
        page.setFont(self.font + '-Bold', 8)
        for column in self.columns:
            page.drawString(x, y, column.label)
            x += column.width

    def build(self, rows):

        buffer = BytesIO()
        page = NumberedPageCanvas(buffer)
        page.setPageSize(landscape(letter))

        y = 580
        page.setFont(self.font + '-Bold', 20)
        page.drawString(10, y, self.title)
        y -= 20

        self.header(page, 20, 560)

        count = 0

        y = 540
        page.setFont(self.font, 8)
        for row in rows:

            values = []
            for column in self.grid.column_set.all():
                field = self.get_field(column.label)
                if field:
                    values.append(str(row[column.label] if self.grid.group_by else value(row, column.label)))

            i = 0
            max_height = 0
            for column in self.columns:
                height = self.get_height(values[i], self.font, 8, column.width)
                if height > max_height:
                    max_height = height
                i += 1

            if y - max_height < 40:
                page.showPage()
                self.header(page, 20, 580)
                page.setFont(self.font, 8)
                y = 560

            if count % 2 == 0:
                page.setStrokeAlpha(0)
                page.setFillColor(self.list_color)
                page.rect(10, y+self.font_size-max_height, 760, max_height, fill=1)
                page.setFillColorRGB(0, 0, 0, 1)
            count += 1

            x = 20
            i = 0
            for column in self.columns:
                self.wrap(page, x, y, values[i], self.font, 8, column.width)
                x += column.width
                i += 1

            y -= max_height
            y -= self.line_padding

        page.showPage()
        page.save()

        pdf = buffer.getvalue()
        buffer.close()
        return pdf



# Generated by Django 2.2 on 2022-06-10 05:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0005_grid_filter'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='column',
            name='criteria',
        ),
        migrations.RemoveField(
            model_name='column',
            name='hidden',
        ),
        migrations.RemoveField(
            model_name='column',
            name='value',
        ),
        migrations.AddField(
            model_name='column',
            name='sort',
            field=models.IntegerField(default=0),
        ),
    ]



# Generated by Django 2.2 on 2022-05-12 21:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0004_auto_20220212_1620'),
    ]

    operations = [
        migrations.AddField(
            model_name='grid',
            name='filter',
            field=models.CharField(blank=True, default='', max_length=255, null=True),
        ),
    ]



# Generated by Django 2.2 on 2022-02-11 07:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Grid',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('table', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('comment', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('filter_by', models.CharField(blank=True, default='', max_length=512, null=True)),
                ('group_by', models.BooleanField(default=False)),
                ('distinct', models.BooleanField(default=False)),
                ('show_on_home', models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterField(
            model_name='log',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='actor', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='Column',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('value', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('criteria', models.IntegerField(default=1)),
                ('width', models.IntegerField(blank=True, default=80, null=True)),
                ('order', models.IntegerField(default=0)),
                ('hidden', models.BooleanField(default=False)),
                ('grid', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.Grid')),
            ],
        ),
    ]
